﻿using ProjectGasolineTanker.Exceptions;
namespace ProjectGasolineTanker.CollectionGenericObjects;

/// <summary>
/// Параметризованный набор объектов
/// </summary>
/// <typeparam name="T">Параметр: ограничение - ссылочный тип</typeparam>
public class MassiveGenericObjects<T> : ICollectionGenericObjects<T>
    where T : class
{
    /// <summary>
    /// Массив объектов, которые храним
    /// </summary>
    private T?[] _collection;

    public int Count => _collection.Length;

    public int MaxCount
    {
        get
        {
            return _collection.Length;
        }
        set
        {
            if (value > 0)
            {
                if (Count > 0)
                {
                    Array.Resize(ref _collection, value);
                }
                else
                {
                    _collection = new T?[value];
                }
            }
        }
        
    }

    public CollectionType GetCollectionType => CollectionType.Massive;

    /// <summary>
    /// Конструктор
    /// </summary>
    public MassiveGenericObjects()
    {
        _collection = Array.Empty<T?>();
    }

    public T Get(int position)
    {
        // проверка позиции
        // выброс ошибки, если выход за границы массива
        //выброс ошибки, если объект пустой
        if (position >= Count || position < 0) throw new PositionOutOfCollectionException(position);
        if (_collection[position] == null) throw new ObjectNotFoundException(position);
        return _collection[position];
        
    }
    public int Insert(T obj)
    {
        // вставка в свободное место набора
        // выброс ошибки, если переполнение
        //выброс ошибки, если выход за границы массива
        for (int i = 0; i < Count; i++)
        {
            if (_collection[i] == null)
            {
                _collection[i] = obj;
                return i;
            }
        }
        throw new CollectionOverflowException(Count);

    }
    public int Insert(T obj, int position)
    {
        // проверка позиции
        // проверка, что элемент массива по этой позиции пустой, если нет, то
        // ищется свободное место после этой позиции и идет вставка туда, если нет после, ищем до
        // вставка
        //выброс ошибки, если переполнение
        //выброс ошибки, если выход за границы массива
        if (position >= Count || position < 0) throw new PositionOutOfCollectionException(position);

        if (_collection[position] == null)
        {
            _collection[position] = obj;
            return position;
        }
        else
        {
            for (int i = 1; i < Count; ++i)
            {
                if (_collection[position + i] == null)
                {
                    _collection[position + i] = obj;
                    return position + i;
                }
                for (i = position - 1; i >= 0; i--)
                {
                    if (_collection[i] == null)
                    {
                        _collection[i] = obj;
                        return i;
                    }
                }
            }
        }
        throw new CollectionOverflowException(Count);

    }
    public T Remove(int position)
    {
        //// проверка позиции
        //// удаление объекта из массива, присвоив элементу массива значение null
        // выброс ошибки, если выход за границы массива
        // выброс ошибки, если объект пустой
        if (position >= Count || position < 0) throw new PositionOutOfCollectionException(position);
        if (_collection[position] == null) throw new ObjectNotFoundException(position);
        T temp = _collection[position];
        _collection[position] = null;
        return temp;

    }


    public IEnumerable<T?> GetItems()
    {
        for (int i = 0; i < _collection.Length; ++i)
        {
            yield return _collection[i];
        }
    }
}
