﻿namespace ProjectGasolineTanker.Drawings;
using ProjectGasolineTanker.Entities;
/// <summary>
/// Расширение для класса EntityTruck
/// </summary>

public static class ExtentionDrawingTruck
{
    /// <summary>
    /// Разделитель для записи информации по объекту в файл
    /// </summary>
    private static readonly string _separatorForObject = ":";

    /// <summary>
    /// Создание объекта из строки
    /// </summary>
    /// <param name="info">Строка с данными для создания объекта</param>
    /// <returns>Объект</returns>

    public static DrawingTruck? CreateDrawingTruck(this string info)
    {
        string[] strs = info.Split(_separatorForObject);
        EntityTruck? truck = EntityGasolineTanker.CreateEntityGasolineTanker(strs);
        if (truck != null)
        {
            return new DrawingGasolineTanker((EntityGasolineTanker)truck);
        }
        truck = EntityTruck.CreateEntityTruck(strs);
        if (truck != null)
        {
            return new DrawingTruck(truck);
        }
        return null;
    }

    /// <summary>
    /// Получение данных для сохранения в файл
    /// </summary>
    /// <param name="drawningTruck">Сохраняемый объект</param>
    /// <returns>Строка с данными по объекту</returns>
    public static string GetDataForSave(this DrawingTruck drawingTruck)
    {
        string[]? array = drawingTruck?.EntityTruck?.GetStringRepresentation();
        if (array == null)
        {
            return string.Empty;
        }
        return string.Join(_separatorForObject, array);
    }
}
